@@ -21,9 +21,3 @@
 
 int __ulp_asunsafe_trylock(void);
 int __ulp_asunsafe_unlock(void);
-
-void *get_loaded_symbol_addr(const char *, const char *, void *);
-
-void *get_loaded_library_base_addr(const char *);
-
-int get_loaded_library_tls_index(const char *);
