@@ -22,11 +22,13 @@
 #ifndef _ULP_LIB_COMMON_
 #define _ULP_LIB_COMMON_
 
+#include "config.h"
 #include <elf.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <time.h>
+#include <sys/types.h>
 
 #define OUT_PATCH_NAME "metadata.ulp"
 #define OUT_REVERSE_NAME "reverse.ulp"
@@ -35,7 +37,6 @@
  * scenarios.  */
 #define ULP_METADATA_BUF_LEN (512 * 1024)
 #define ULP_PATH_LEN 256
-#define RED_ZONE_LEN 128
 
 #define ARRAY_LENGTH(v) (sizeof(v) / sizeof(*(v)))
 
