@@ -26,7 +26,7 @@ main(void)
       printf("Reached the end of file; quitting.\n");
       return 0;
     }
-    int_params(1, 2, 3, 4, 5, 6, 7, 8);
+    int_params(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
     float_params(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
   }
 
