@@ -26,7 +26,7 @@ child = testsuite.spawn('parameters')
 child.expect('Waiting for input.')
 
 child.sendline('')
-child.expect('1-2-3-4-5-6-7-8');
+child.expect('1-2-3-4-5-6-7-8-9-10');
 child.expect('1.0-2.0-3.0-4.0-5.0-6.0-7.0-8.0-9.0-10.0');
 
 try:
