@@ -26,13 +26,13 @@ child = testsuite.spawn('process', script=False)
 child.expect('Waiting for input.')
 
 child.sendline('')
-child.expect('1-2-3-4-5-6-7-8');
+child.expect('1-2-3-4-5-6-7-8-9-10');
 child.expect('1.0-2.0-3.0-4.0-5.0-6.0-7.0-8.0-9.0-10.0');
 
 child.livepatch('.libs/libprocess_livepatch1.so')
 
 child.sendline('')
-child.expect('8-7-6-5-4-3-2-1', reject='1-2-3-4-5-6-7-8');
+child.expect('10-9-8-7-6-5-4-3-2-1', reject='1-2-3-4-5-6-7-8-9-10');
 child.expect('10.0-9.0-8.0-7.0-6.0-5.0-4.0-3.0-2.0-1.0',
              reject='1.0-2.0-3.0-4.0-5.0-6.0-7.0-8.0-9.0-10.0');
 
