@@ -29,40 +29,13 @@
 /** PID of target process in which we will execute the instructions.  */
 static int remote_pid;
 
-/** @brief Interpret a print instruction.
- *
- * @param insn    Instruction to interpet. Must be a print instruction.
- *
- * @return        Size of interpreted instruction.
- */
-static int
-insn_interpret_print(struct ulp_insn *insn)
-{
-  struct ulp_insn_print *p_insn = (struct ulp_insn_print *)insn;
-
-  printf("%s\n", p_insn->bytes);
-  return insn->size;
-}
-
-/** @brief Interpret NOP instruction.
- *
- * @param insn    Instruction to interpet. Must be a NOP instruction.
- *
- * @return        Size of interpreted instruction (always 1 byte).
- */
-static int
-insn_interpret_nop(struct ulp_insn *insn)
-{
-  return sizeof(*insn);
-}
-
-/** @brief Interpret NOP instruction.
+/** @brief Interpret WRITE instruction.
  *
  * @param insn    Instruction to interpet. Must be a WRITE instruction.
  *
  * @return        Size of interpreted instruction.
  */
-static int
+int
 insn_interpret_write(struct ulp_insn *insn)
 {
   int pid = remote_pid; // Pass process pid.
@@ -74,75 +47,6 @@ insn_interpret_write(struct ulp_insn *insn)
   return insn->size;
 }
 
-/** Table of decoders.  Index must match the `enum ulp_insn_table` object.  */
-static int (*decoders[ULP_NUM_INSNS])(struct ulp_insn *insn) = {
-  insn_interpret_nop,
-  insn_interpret_print,
-  insn_interpret_write,
-};
-
-/** @brief Interpret the given instruction.
- *
- * This function will interpret the given instruction.
- *
- * @param insn      Instruction to interpret.
- *
- * @return          Size of instruction interpreted.
- */
-int
-insn_interpret(struct ulp_insn *insn)
-{
-  int index = (int)insn->type;
-  return (decoders[index])(insn);
-}
-
-/** @brief Interpret the instructions in queue.
- *
- * Interpret all instructions inserted into the queue object.
- *
- * @param queue
- */
-int
-insnq_interpret(insn_queue_t *queue)
-{
-  int pc = 0; /* Like a CPU program counter.  */
-  int num_insns_executed = 0;
-
-  int size = queue->size;
-  int num_insns = queue->num_insns;
-  char *buffer = queue->buffer;
-
-  while (num_insns_executed < num_insns) {
-    struct ulp_insn *insn = (struct ulp_insn *)&buffer[pc];
-    if (ulp_insn_valid(insn)) {
-      pc += insn_interpret(insn);
-      num_insns_executed++;
-    }
-    else {
-      /* Abort if an invalid insn is received.  */
-      WARN("on pid: %d invalid insn with opcode %d. Further insns will be "
-           "ignored.",
-           remote_pid, (int)insn->type);
-      return EINSNQ;
-    }
-  }
-
-  /* The pc should stop at the size of the queue.  */
-  if (pc != size) {
-    WARN("on pid %d: there are bytes left in the instruction queue",
-         remote_pid);
-    return EINSNQ;
-  }
-
-  /* Number of instructions should match what is in the queue.  */
-  if (num_insns_executed != num_insns) {
-    WARN("on pid %d: not all instructions executed", remote_pid);
-    return EINSNQ;
-  }
-
-  return 0;
-}
-
 /** @brief Get the instruction queue on remote process.
  *
  * This function will get the instruction queue on `queue_adddr` of process
@@ -271,8 +175,7 @@ insnq_interpret_from_process_(int pid, Elf64_Addr queue_addr)
   static insn_queue_t queue;
 
   if (queue_addr == 0) {
-    /* Libpulp is old and do not have a instruction queue, or is too new and
-       the instruction queue was overcame.  */
+    /* Libpulp is old and do not have a instruction queue.  */
     return EOLDLIBPULP;
   }
 
