@@ -86,6 +86,17 @@ void
 ulp_warn(const char *format, ...)
 {
   if (!ulp_quiet) {
+    time_t rawtime;
+    struct tm *timeinfo, local_tm;
+    char timestamp[32];
+
+    time(&rawtime);
+    timeinfo = localtime_r(&rawtime, &local_tm);
+
+    strftime(timestamp, 32, "[%Y-%m-%d %H:%M:%S] ", timeinfo);
+
+    fputs(timestamp, stderr);
+
     va_list args;
     va_start(args, format);
     vfprintf(stderr, format, args);
@@ -97,6 +108,17 @@ void
 ulp_debug(const char *format, ...)
 {
   if (ulp_verbose) {
+    time_t rawtime;
+    struct tm *timeinfo, local_tm;
+    char timestamp[32];
+
+    time(&rawtime);
+    timeinfo = localtime_r(&rawtime, &local_tm);
+
+    strftime(timestamp, 32, "[%Y-%m-%d %H:%M:%S] ", timeinfo);
+
+    fputs(timestamp, stderr);
+
     va_list args;
     va_start(args, format);
     vfprintf(stderr, format, args);
@@ -495,6 +517,74 @@ get_dynobj_elf_by_cache(struct ulp_process *process)
 }
 #endif // ENABLE_DLINFO_CACHE
 
+
+/** @brief Parsers /proc/pid/maps to figure the EHDR mapping of dynamic object.
+ *
+ * This function reads the /proc/pid/maps to find the EHDR mapping of the
+ * dynamic object.  Usually this can be found by looking at the link_map.l_addr
+ * field, however for the main process this information is not available there.
+ *
+ * @param p   process to analyze.
+ * @param obj Object representing a library, or the main program itself, from p.
+ *
+ * @return 0 on success, anything else on failure.
+ */
+ElfW(Addr)
+get_ehdr_addr(struct ulp_process *p, struct ulp_dynobj *obj)
+{
+  if (obj->link_map.l_addr) {
+    return obj->link_map.l_addr;
+  }
+
+  const char *format_str = "/proc/%d/maps";
+  char filename[strlen(format_str) + 10];
+  sprintf(filename, format_str, p->pid);
+
+  FILE *maps = fopen(filename, "r");
+  if (maps == NULL) {
+    DEBUG("error: unable to open maps.");
+    return 0;
+  }
+
+  char *line = NULL;
+  size_t len = 0;
+  ssize_t n;
+
+  while ((n = getline(&line, &len, maps)) != -1) {
+    const char *str_addr1  =    strtok(line, "-");
+    /* clang-format off */
+
+ /* const char *str_addr2  = */ strtok(NULL, " ");
+ /* const char *str_perm   = */ strtok(NULL, " ");
+ /* const char *str_offset = */ strtok(NULL, " ");
+ /* const char *str_dev    = */ strtok(NULL, " ");
+ /* const char *str_inode  = */ strtok(NULL, " ");
+    const char *str_path   =    strtok(NULL, " ");
+
+    /* clang-format on */
+
+    /* Get the basename (last string after all /) and remove the newline there
+       if it exists.  */
+    char *bin_name = (char *) get_basename(str_path);
+    unsigned bin_len = strlen(bin_name);
+    if (bin_name[bin_len-1] == '\n') {
+      bin_name[bin_len-1] = '\0';
+    }
+
+    if (strcmp(bin_name, p->dynobj_main->filename) == 0) {
+      ElfW(Addr) addr1 = strtoul(str_addr1, NULL, 16);
+      fclose(maps);
+      FREE_AND_NULLIFY(line);
+      return addr1;
+    }
+  }
+
+  fclose(maps);
+  FREE_AND_NULLIFY(line);
+
+  return 0UL;
+}
+
 /** @brief Parses ELF headers of dynobj `obj` from process with pid `pid`.
  *
  * This function read the remote process memory to locate the following
@@ -511,8 +601,12 @@ get_dynobj_elf_by_cache(struct ulp_process *process)
  * @return 0 on success, anything else on failure.
  */
 int
-parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
+parse_dynobj_elf_headers(struct ulp_process *p, struct ulp_dynobj *obj)
 {
+  pid_t pid = p->pid;
+
+  ElfW(Addr) l_addr = 0;
+
   ElfW(Addr) ehdr_addr = 0;
   ElfW(Ehdr) ehdr;
   ElfW(Addr) phdr_addr = 0;
@@ -537,18 +631,10 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
     return ENOLINKMAP;
   }
 
-  /* l_addr holds the pointer to the ELF header.  */
-  ehdr_addr = obj->link_map.l_addr;
+  l_addr = obj->link_map.l_addr;
+  ehdr_addr = get_ehdr_addr(p, obj);
 
-  /* Read ELF header from remote process.  */
-  if (ehdr_addr == 0) {
-    /* If l_addr is zero, it means that there is no load bias.  In that case,
-     * the elf address is on address 0x400000 on x86_64.  */
-    ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, 0x400000UL);
-  }
-  else {
-    ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, ehdr_addr);
-  }
+  ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, ehdr_addr);
   if (ret != 0) {
     DEBUG("Unable to read ELF header from process %d\n", pid);
     return ETARGETHOOK;
@@ -562,8 +648,6 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
 
   /* Get first process header address.  */
   phdr_addr = ehdr_addr + ehdr.e_phoff;
-  if (ehdr_addr == 0)
-    phdr_addr += 0x400000UL;
 
   /* Iterate over each process header.  */
   for (i = 0; i < ehdr.e_phnum; i++) {
@@ -580,7 +664,7 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
     /* Look for the dynamic section.  */
     if (phdr.p_type == PT_DYNAMIC && !pt_dynamic_ran) {
       ElfW(Dyn) dyn;
-      ElfW(Addr) dyn_addr = ehdr_addr + phdr.p_paddr;
+      ElfW(Addr) dyn_addr = l_addr + phdr.p_vaddr;
 
       /* Iterate over each tag in this section.  */
       do {
@@ -625,14 +709,18 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
     }
     else if (phdr.p_type == PT_NOTE) {
       /* We are after the build id.  */
-
-      ElfW(Addr) note_addr = ehdr_addr + phdr.p_paddr;
+      ElfW(Addr) note_addr = l_addr + phdr.p_vaddr;
       unsigned sec_size = phdr.p_memsz;
       ElfW(Addr) note_addr_end = note_addr + sec_size;
 
       do {
         ElfW(Nhdr) note;
 
+        /* Stop processing more NOTES headers if Build ID already found. */
+        if (pt_note_ran == true) {
+          break;
+        }
+
         /* Get the note section in remote process.  */
         ret = read_memory((char *)&note, sizeof(note), pid, note_addr);
         if (ret != 0) {
@@ -1088,7 +1176,7 @@ parse_main_dynobj(struct ulp_process *process)
     return ret;
   }
 
-  parse_dynobj_elf_headers(process->pid, obj);
+  parse_dynobj_elf_headers(process, obj);
 
   return 0;
 }
@@ -1129,8 +1217,9 @@ parse_lib_dynobj(struct ulp_dynobj *obj, struct ulp_process *process)
     return 0;
 
   /* Pointers to linux-vdso.so are invalid, so skip this library.  */
-  if (strcmp(obj->filename, "linux-vdso.so.1"))
-    parse_dynobj_elf_headers(pid, obj);
+  if (strcmp(obj->filename, "linux-vdso.so.1") &&
+      strcmp(obj->filename, "linux-vdso64.so.1"))
+    parse_dynobj_elf_headers(process, obj);
 
   /* Only libpulp.so should have those symbols exported.  */
   if (strstr(libname, "libpulp.so")) {
@@ -1533,7 +1622,7 @@ patch_applied(struct ulp_process *process, unsigned char *id, int *result)
 {
   int ret;
   struct ulp_thread *thread;
-  struct user_regs_struct context;
+  registers_t context;
   ElfW(Addr) routine;
 
   DEBUG("checking if live patch is already applied.");
@@ -1561,7 +1650,7 @@ patch_applied(struct ulp_process *process, unsigned char *id, int *result)
   if (ret)
     return ret;
 
-  *result = context.rax;
+  *result = FUNCTION_RETURN_REG(context);
   return 0;
 }
 
@@ -1577,11 +1666,12 @@ patch_applied(struct ulp_process *process, unsigned char *id, int *result)
  * called after successful thread hijacking.
  */
 int
-apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
+apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size,
+            bool disable_seccomp_p)
 {
   int ret;
   struct ulp_thread *thread;
-  struct user_regs_struct context;
+  registers_t context;
   ElfW(Addr) routine;
 
   struct ulp_dynobj *dynobj_libpulp = process->dynobj_libpulp;
@@ -1599,26 +1689,33 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
     return ETARGETHOOK;
   }
 
+  if (disable_seccomp_p) {
+    if (disable_seccomp(process->pid)) {
+      WARN("unable to disable seccomp in target process: %s", libpulp_strerror(errno));
+      return errno;
+    }
+  }
+
   thread = process->main_thread;
   context = thread->context;
   routine = dynobj_libpulp->trigger;
 
-  rax = context.rax;
+  rax = FUNCTION_RETURN_REG(context);
 
   DEBUG(">>> running libpulp functions within target process...");
   ret = run_and_redirect(thread->tid, &context, routine);
   if (ret) {
     WARN("error during live patch application: %s",
-         libpulp_strerror(context.rax));
+         libpulp_strerror(FUNCTION_RETURN_REG(context)));
   }
   DEBUG(">>> done.");
   if (ret)
     return ret;
 
-  if (context.rax != 0) {
-    if (context.rax == EAGAIN)
+  if (FUNCTION_RETURN_REG(context) != 0) {
+    if (FUNCTION_RETURN_REG(context) == EAGAIN)
       DEBUG("patching failed in libpulp.so: libc/libdl locks were busy");
-    else if (context.rax == rax) {
+    else if (FUNCTION_RETURN_REG(context) == rax) {
       /* If rax register is not changed in this process, it is evidence that
          the routine in libpulp.so wasn't executed by some reason.  */
       DEBUG("patching failed in libpulp.so: %s",
@@ -1627,7 +1724,7 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
     }
     else
       DEBUG("patching failed in libpulp.so: %s",
-            libpulp_strerror(context.rax));
+            libpulp_strerror(FUNCTION_RETURN_REG(context)));
   }
 
   /* Interpret now any code that libpulp send to us.  */
@@ -1637,7 +1734,7 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
     return ret;
   }
 
-  return context.rax;
+  return FUNCTION_RETURN_REG(context);
 }
 
 int
@@ -1645,7 +1742,7 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
 {
   int ret;
   struct ulp_thread *thread;
-  struct user_regs_struct context;
+  registers_t context;
   ElfW(Addr) routine;
 
   DEBUG("beginning patches removal.");
@@ -1681,13 +1778,13 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
   if (ret)
     return ret;
 
-  if (context.rax != 0) {
-    if (context.rax == EAGAIN)
+  if (FUNCTION_RETURN_REG(context) != 0) {
+    if (FUNCTION_RETURN_REG(context) == EAGAIN)
       DEBUG("patches reverse-all failed in libpulp.so: libc/libdl locks were "
             "busy");
     else
       DEBUG("patches reverse-all failed in libpulp.so: %s",
-            libpulp_strerror(context.rax));
+            libpulp_strerror(FUNCTION_RETURN_REG(context)));
   }
 
   /* Interpret now any code that libpulp send to us.  */
@@ -1697,7 +1794,7 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
     return ret;
   }
 
-  return context.rax;
+  return FUNCTION_RETURN_REG(context);
 }
 
 /* Reads the global universe counter in PROCESS. Returns the
@@ -1707,7 +1804,7 @@ int
 read_global_universe(struct ulp_process *process)
 {
   struct ulp_thread *thread;
-  struct user_regs_struct context;
+  registers_t context;
   ElfW(Addr) routine;
 
   if (!process->all_threads_hijacked) {
@@ -1724,7 +1821,7 @@ read_global_universe(struct ulp_process *process)
     return -1;
   };
 
-  process->global_universe = context.rax;
+  process->global_universe = FUNCTION_RETURN_REG(context);
   return 0;
 }
 
@@ -1824,8 +1921,10 @@ extract_ulp_from_so_to_mem(const char *livepatch, bool revert, char **out,
 
   /* Get full path to patch buffer.  */
   if (realpath(livepatch, &path_buffer[path_size]) == NULL) {
-    WARN("Unable to retrieve realpath to %s", livepatch);
-    return 0;
+    /* If we can't figure out the realpath, then use the path it was given to
+       us.  */
+    DEBUG("Unable to retrieve realpath to %s: %s", livepatch, libpulp_strerror(errno));
+    strcpy(&path_buffer[path_size], livepatch);
   }
   path_size = strlen(path_buffer) + 1;
 
@@ -1845,6 +1944,9 @@ extract_ulp_from_so_to_mem(const char *livepatch, bool revert, char **out,
   char *final_meta = (char *)malloc(meta_size);
   char *meta_head = final_meta;
 
+
+  /* FIXME: This is absurdly awkward.  */
+
   /* Copy the final metadata into final_meta buffer.  Things works here as
    * follows:
    *
